@@ -2,10 +2,11 @@
 This does not work with "multi-embedding" representations like CP Word or Octuple.
 """
 
-from typing import List, Dict, Union, Any, Type
+from typing import List, Dict, Union, Any, Type, Tuple
 from pathlib import Path, PurePath
 import json
 from random import choices
+from copy import deepcopy
 
 from miditoolkit import MidiFile
 from tqdm import tqdm
@@ -48,7 +49,8 @@ def bpe(tokenizer: Type[MIDITokenizer], *args, **kwargs):
                 out_dir = Path(out_dir)
             out_dir.mkdir(parents=True, exist_ok=True)
             files_paths = list(Path(tokens_path).glob('**/*.json'))
-            files_paths_bpe = choices(files_paths, k=files_lim) if files_lim is not None else files_paths
+            files_paths_bpe = choices(files_paths, k=files_lim) if \
+                (files_lim is not None and files_lim > len(files_paths)) else files_paths
             samples = []
             samples_paths = []
             original_lengths = []
@@ -61,7 +63,7 @@ def bpe(tokenizer: Type[MIDITokenizer], *args, **kwargs):
                 original_lengths += [len(track) for track in file['tokens']]
 
             # Byte Pair Encoding
-            pbar = tqdm(total=vocab_size - len(self.vocab), desc='Running byte pair encoding')
+            pbar = tqdm(total=vocab_size - len(self.vocab), desc='Learning byte pair encoding')
             while len(self.vocab) < vocab_size:
                 pbar.update(1)
                 occurrences = {}  # count occurrences of successive tokens
@@ -124,20 +126,35 @@ def bpe(tokenizer: Type[MIDITokenizer], *args, **kwargs):
             if not self.has_bpe:
                 return tokens
 
-            previous_len = len(tokens) + 1  # + 1 to fool when entering the loop
-            while previous_len != len(tokens):
-                previous_len = len(tokens)
-                # loop over BPE tokens from the vocabulary
-                for tok, token_succession in self.bpe_successions.items():
-                    i = 0  # will check if any token succession from the input token sequence matches the BPE
-                    while i <= len(tokens) - len(token_succession):
-                        if tokens[i:i + len(token_succession)] == token_succession:
-                            tokens[i] = tok  # if so, will replace the current token and remove the next ones to replace
-                            for _ in range(len(token_succession) - 1):
-                                del tokens[i + 1]
-                        i += 1
+            previous_len = len(tokens) + 1  # + 1 to fool when entering the loop the first time
+            while previous_len != len(tokens):  # if this is True, it means no more BPE combinations is possible
+                previous_len = len(tokens)  # length of the token sequence before applying BPE
+                for tok, token_succession in self.bpe_successions.items():  # loops over BPE tokens from the vocabulary
+                    occurrences = self.__subfind(tokens, token_succession)
+                    for idx in reversed(occurrences):
+                        tokens[idx] = tok
+                        for _ in range(len(token_succession) - 1):
+                            del tokens[idx + 1]
             return tokens
 
+        @staticmethod
+        def __subfind(in_list: List[int], pattern: List[int]) -> List[int]:
+            """Finds the locations of a pattern within a list.
+            Adapted from: https://stackoverflow.com/questions/10106901/elegant-find-sub-list-in-list
+            Related: https://www.reddit.com/r/learnpython/comments/2xqlwj/using_npwhere_to_find_subarrays/
+            After testing, the numba jit version does not seem to be much faster.
+            The conversion of python lists to numba.typed.List() seems to also take time.
+
+            :param in_list: input list to analyze
+            :param pattern: pattern to detect
+            :return: indices of in_list where the pattern has been found
+            """
+            matches = []
+            for i in range(len(in_list)):
+                if in_list[i] == pattern[0] and in_list[i:i + len(pattern)] == pattern:
+                    matches.append(i)
+            return matches
+
         def apply_bpe_to_dataset(self, dataset_path: Union[Path, PurePath, str], out_path: Union[Path, PurePath, str]):
             r"""Apply BPE to an already tokenized dataset (with no BPE).
 
@@ -147,7 +164,7 @@ def bpe(tokenizer: Type[MIDITokenizer], *args, **kwargs):
             if not self.has_bpe:
                 return
 
-            files_paths = list(Path(dataset_path).glob(f'**/*.json'))
+            files_paths = list(Path(dataset_path).glob('**/*.json'))
             for path in tqdm(files_paths, desc='Applying BPE to dataset'):
                 sample = self.load_tokens(path)
                 sample_bpe = [self.apply_bpe(track) for track in sample['tokens']]
@@ -162,18 +179,22 @@ def bpe(tokenizer: Type[MIDITokenizer], *args, **kwargs):
             tokens = super().midi_to_tokens(midi, *args_, **kwargs_)
             return [self.apply_bpe(track) for track in tokens] if self.has_bpe else tokens
 
-        def tokens_to_events(self, tokens: List[int], **kwargss) -> List[Event]:
+        def tokens_to_events(self, tokens: List[int], return_decomposed_tokens: bool = False, **kwargss) \
+                -> Union[List[Event], Tuple[List[Event], List[int]]]:
             r"""First decomposes BPE tokens, then converts them in their respective event objects.
 
             :param tokens: sequence of tokens to convert
+            :param return_decomposed_tokens: set True if you want the decomposed tokens to be returned (default: False)
             :return: the sequence of corresponding events
             """
             # Decompose BPE tokens first
-            tokens = self.decompose_bpe(tokens)
-            return super().tokens_to_events(tokens, **kwargss)
+            decomposed_tokens = self.decompose_bpe(deepcopy(tokens))
+            events = super().tokens_to_events(decomposed_tokens, **kwargss)
+            return (events, decomposed_tokens) if return_decomposed_tokens else events
 
         def decompose_bpe(self, tokens: List[int]) -> List[int]:
             r"""Decomposes a sequence of tokens containing BP encoded tokens into "prime" tokens.
+            It is an inplace operation.
 
             :param tokens: token sequence to decompose
             :return: decomposed token sequence
@@ -227,6 +248,7 @@ def bpe(tokenizer: Type[MIDITokenizer], *args, **kwargs):
                            'beat_res': {f'{k1}_{k2}': v for (k1, k2), v in self.beat_res.items()},
                            'nb_velocities': len(self.velocities),
                            'additional_tokens': self.additional_tokens,
+                           '_pad': self._pad,
                            '_sos_eos': self._sos_eos,
                            '_mask': self._mask,
                            'encoding': f'{getmro(self.__class__)[1].__name__}_bpe',
