@@ -11,7 +11,8 @@ from miditoolkit import Instrument, Note, TempoChange
 from .midi_tokenizer_base import MIDITokenizer
 from .vocabulary import Vocabulary, Event
 from .utils import detect_chords
-from .constants import *
+from .constants import PITCH_RANGE, NB_VELOCITIES, BEAT_RES, ADDITIONAL_TOKENS, TIME_DIVISION, TEMPO, \
+    MIDI_INSTRUMENTS, CHORD_MAPS
 
 
 class MIDILike(MIDITokenizer):
@@ -33,16 +34,19 @@ class MIDILike(MIDITokenizer):
             The values are the resolution, in samples per beat, of the given range, ex 8
     :param nb_velocities: number of velocity bins
     :param additional_tokens: specifies additional tokens (chords, time signature, rests, tempo...)
-    :param sos_eos_tokens: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary
+    :param pad: will include a PAD token, used when training a model with batch of sequences of
+            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
+    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
+            (default: False)
     :param mask: will add a MASK token to the vocabulary (default: False)
     :param params: can be a path to the parameter (json encoded) file or a dictionary
     """
 
     def __init__(self, pitch_range: range = PITCH_RANGE, beat_res: Dict[Tuple[int, int], int] = BEAT_RES,
                  nb_velocities: int = NB_VELOCITIES, additional_tokens: Dict[str, bool] = ADDITIONAL_TOKENS,
-                 sos_eos_tokens: bool = False, mask: bool = False, params=None):
+                 pad: bool = True, sos_eos: bool = False, mask: bool = False, params=None):
         additional_tokens['TimeSignature'] = False  # not compatible
-        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, sos_eos_tokens, mask, params)
+        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, pad, sos_eos, mask, params)
 
     def track_to_tokens(self, track: Instrument) -> List[int]:
         r"""Converts a track (miditoolkit.Instrument object) into a sequence of tokens
@@ -62,15 +66,15 @@ class MIDILike(MIDITokenizer):
         # Creates the Note On, Note Off and Velocity events
         for n, note in enumerate(track.notes):
             # Note On
-            events.append(Event(type_='Note-On', time=note.start, value=note.pitch, desc=note.end))
+            events.append(Event(type_='Note-On', value=note.pitch, time=note.start, desc=note.end))
             # Velocity
-            events.append(Event(type_='Velocity', time=note.start, value=note.velocity, desc=f'{note.velocity}'))
+            events.append(Event(type_='Velocity', value=note.velocity, time=note.start, desc=f'{note.velocity}'))
             # Note Off
-            events.append(Event(type_='Note-Off', time=note.end, value=note.pitch, desc=note.end))
+            events.append(Event(type_='Note-Off', value=note.pitch, time=note.end, desc=note.end))
         # Adds tempo events if specified
         if self.additional_tokens['Tempo']:
             for tempo_change in self.current_midi_metadata['tempo_changes']:
-                events.append(Event(type_='Tempo', time=tempo_change.time, value=tempo_change.tempo,
+                events.append(Event(type_='Tempo', value=tempo_change.tempo, time=tempo_change.time,
                                     desc=tempo_change.tempo))
 
         # Sorts events
@@ -94,30 +98,30 @@ class MIDILike(MIDITokenizer):
                 rest_tick = previous_tick  # untouched tick value to the order is not messed after sorting
 
                 if rest_beat > 0:
-                    events.append(Event(type_='Rest', time=rest_tick, value=f'{rest_beat}.0',
+                    events.append(Event(type_='Rest', value=f'{rest_beat}.0', time=rest_tick,
                                         desc=f'{rest_beat}.0'))
                     previous_tick += rest_beat * self.current_midi_metadata['time_division']
 
                 while rest_pos >= self.rests[0][1]:
                     rest_pos_temp = min([r[1] for r in self.rests], key=lambda x: abs(x - rest_pos))
-                    events.append(Event(type_='Rest', time=rest_tick, value=f'0.{rest_pos_temp}',
+                    events.append(Event(type_='Rest', value=f'0.{rest_pos_temp}', time=rest_tick,
                                         desc=f'0.{rest_pos_temp}'))
                     previous_tick += round(rest_pos_temp * ticks_per_sample)
                     rest_pos -= rest_pos_temp
 
-                # Adds an additional time shift if needed
+                # Adds a time-shift if needed
                 if rest_pos > 0:
                     time_shift = round(rest_pos * ticks_per_sample)
                     index = np.argmin(np.abs(dur_bins - time_shift))
-                    events.append(Event(type_='Time-Shift', time=previous_tick,
-                                        value='.'.join(map(str, self.durations[index])), desc=f'{time_shift} ticks'))
+                    events.append(Event(type_='Time-Shift', value='.'.join(map(str, self.durations[index])),
+                                        time=previous_tick, desc=f'{time_shift} ticks'))
 
-            # Time shift
+            # Time-shift
             else:
                 time_shift = event.time - previous_tick
                 index = np.argmin(np.abs(dur_bins - time_shift))
-                events.append(Event(type_='Time-Shift', time=previous_tick,
-                                    value='.'.join(map(str, self.durations[index])), desc=f'{time_shift} ticks'))
+                events.append(Event(type_='Time-Shift', value='.'.join(map(str, self.durations[index])),
+                                    time=previous_tick, desc=f'{time_shift} ticks'))
 
             if event.type == 'Note-On':
                 previous_note_end = max(previous_note_end, event.desc)
@@ -181,7 +185,7 @@ class MIDILike(MIDITokenizer):
                         if duration != 0:
                             instrument.notes.append(Note(vel, pitch, current_tick, current_tick + duration))
                         ei += 1
-                except IndexError as _:
+                except IndexError:
                     pass
             elif events[ei].type == 'Time-Shift':
                 current_tick += self._token_duration_to_ticks(events[ei].value, time_division)
@@ -207,9 +211,9 @@ class MIDILike(MIDITokenizer):
         :return: the vocabulary object
         """
         if sos_eos_tokens is not None:
-            print(f'\033[93msos_eos_tokens argument is depreciated and will be removed in a future update, '
-                  f'_create_vocabulary now uses self._sos_eos attribute set a class init \033[0m')
-        vocab = Vocabulary({'PAD_None': 0}, sos_eos=self._sos_eos, mask=self._mask)
+            print('\033[93msos_eos_tokens argument is depreciated and will be removed in a future update, '
+                  '_create_vocabulary now uses self._sos_eos attribute set a class init \033[0m')
+        vocab = Vocabulary(pad=self._pad, sos_eos=self._sos_eos, mask=self._mask)
 
         # NOTE ON
         vocab.add_event(f'Note-On_{i}' for i in self.pitch_range)
