@@ -15,7 +15,7 @@ from miditoolkit import MidiFile, Instrument, Note, TempoChange, TimeSignature
 
 from .vocabulary import Vocabulary, Event
 from .utils import remove_duplicated_notes, get_midi_programs
-from .constants import TIME_DIVISION
+from .constants import TIME_DIVISION, CURRENT_PACKAGE_VERSION
 
 
 class MIDITokenizer(ABC):
@@ -29,14 +29,17 @@ class MIDITokenizer(ABC):
             The values are the resolution, in samples per beat, of the given range, ex 8
     :param nb_velocities: number of velocity bins
     :param additional_tokens: specifies additional tokens (chords, rests, tempo, time signature...)
-    :param sos_eos_tokens: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary
+    :param pad: will include a PAD token, used when training a model with batch of sequences of
+            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
+    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
+            (default: False)
     :param mask: will add a MASK token to the vocabulary (default: False)
     :param params: can be a path to the parameter (json encoded) file or a dictionary
     """
 
     def __init__(self, pitch_range: range, beat_res: Dict[Tuple[int, int], int], nb_velocities: int,
-                 additional_tokens: Dict[str, Union[bool, int, Tuple[int, int]]], sos_eos_tokens: bool = False,
-                 mask: bool = False, params: Union[str, Path, PurePath, Dict[str, Any]] = None):
+                 additional_tokens: Dict[str, Union[bool, int, Tuple[int, int]]], pad: bool = True,
+                 sos_eos: bool = False, mask: bool = False, params: Union[str, Path, PurePath, Dict[str, Any]] = None):
         # Initialize params
         self.vocab = None
         if params is None:
@@ -44,7 +47,8 @@ class MIDITokenizer(ABC):
             self.beat_res = beat_res
             self.additional_tokens = additional_tokens
             self.nb_velocities = nb_velocities
-            self._sos_eos = sos_eos_tokens
+            self._pad = pad
+            self._sos_eos = sos_eos
             self._mask = mask
         else:
             self.load_params(params)
@@ -77,7 +81,7 @@ class MIDITokenizer(ABC):
             self.time_signatures = self.__create_time_signatures()
 
         # Vocabulary and token types graph
-        if self.vocab is None:  # in case it was already loaded by an overriding load_params method, such as with BPE
+        if self.vocab is None:  # in case it was already loaded by an overridden load_params method, such as with BPE
             self.vocab = self._create_vocabulary()
         self.tokens_types_graph = self._create_token_types_graph()
 
@@ -175,20 +179,20 @@ class MIDITokenizer(ABC):
         :return: the sequence of corresponding events
         """
         if multi_voc is not None:
-            print(f'\033[93mmiditok warning: tokens_to_events method no longer need multi_voc argument as '
-                  f'it is now a class attribute, it will be removed in future updates.\033[0m')
+            print('\033[93mmiditok warning: tokens_to_events method no longer need multi_voc argument as '
+                  'it is now a class attribute, it will be removed in future updates.\033[0m')
         events = []
         if isinstance(self.vocab, list):  # multiple vocabularies
             for multi_token in tokens:
                 multi_event = []
                 for i, token in enumerate(multi_token):
                     name, val = self.vocab[i].token_to_event[token].split('_')
-                    multi_event.append(Event(name, None, val, None))
+                    multi_event.append(Event(name, val))
                 events.append(multi_event)
         else:
             for token in tokens:
                 name, val = self.vocab.token_to_event[token].split('_')
-                events.append(Event(name, None, val, None))
+                events.append(Event(name, val))
         return events
 
     def tokens_to_midi(self, tokens: List[List[Union[int, List[int]]]],
@@ -217,7 +221,8 @@ class MIDITokenizer(ABC):
             if i == 0:  # only keep tempo changes of the first track
                 midi.tempo_changes = tempo_changes
                 midi.tempo_changes[0].time = 0
-        midi.max_tick = max([max([note.end for note in track.notes]) for track in midi.instruments])
+        midi.max_tick = max([max([note.end for note in track.notes]) if len(track.notes) > 0 else 0
+                             for track in midi.instruments])
 
         # Write MIDI file
         if output_path:
@@ -348,22 +353,15 @@ class MIDITokenizer(ABC):
         See other classes (REMI, MIDILike ...) for examples of how to implement it."""
         raise NotImplementedError
 
-    def _add_pad_type_to_graph(self, dic: Dict[str, List[str]]):
-        r"""DEPRECIATED: has been replaced by _add_special_tokens_to_types_graph.
-        This method will call _add_special_tokens_to_types_graph, see below.
-
-        :param dic: token types graph to add PAD type
-        """
-        self._add_special_tokens_to_types_graph(dic)
-
     def _add_special_tokens_to_types_graph(self, dic: Dict[str, List[str]]):
         r"""Inplace adds special tokens (PAD, EOS, SOS, MASK) types to the token types graph dictionary.
 
         :param dic: token types graph to add PAD type
         """
-        for value in dic.values():
-            value.append('PAD')
-        dic['PAD'] = ['PAD']
+        if self._pad:
+            for value in dic.values():
+                value.append('PAD')
+            dic['PAD'] = ['PAD']
         if self._sos_eos:
             dic['SOS'] = list(dic.keys())
             dic['EOS'] = []
@@ -439,7 +437,7 @@ class MIDITokenizer(ABC):
         """
         max_beat_res, nb_notes = self.additional_tokens.get('time_signature_range', (4, 1))
         assert max_beat_res > 0 and math.log2(max_beat_res).is_integer(), \
-            f'The beat resolution in time signature must be a power of 2'
+            'The beat resolution in time signature must be a power of 2'
 
         time_signatures = []
         for i in range(0, int(math.log2(max_beat_res)) + 1):  # 1 ~ max_beat_res
@@ -517,7 +515,7 @@ class MIDITokenizer(ABC):
                 if logging:
                     print(f'File not found: {midi_path}')
                 continue
-            except Exception as _:  # ValueError, OSError, FileNotFoundError, IOError, EOFError, mido.KeySignatureError
+            except Exception:  # ValueError, OSError, FileNotFoundError, IOError, EOFError, mido.KeySignatureError
                 continue
 
             # Checks the time division is valid
@@ -561,16 +559,18 @@ class MIDITokenizer(ABC):
         return err / len(tokens)
 
     @staticmethod
-    def save_tokens(tokens, path: Union[str, Path, PurePath], programs: List[Tuple[int, bool]] = None):
+    def save_tokens(tokens, path: Union[str, Path, PurePath], programs: List[Tuple[int, bool]] = None, **kwargs):
         r"""Saves tokens as a JSON file.
+        Use kwargs to save any additional information within the JSON file.
 
         :param tokens: tokens, as any format
         :param path: path of the file to save
         :param programs: (optional), programs of the associated tokens, should be
                         given as a tuples (int, bool) for (program, is_drum)
+        :param kwargs: any additional information to save within the JSON file.
         """
         with open(path, 'w') as outfile:
-            json.dump({'tokens': tokens, 'programs': programs if programs is not None else []}, outfile)
+            json.dump({'tokens': tokens, 'programs': programs if programs is not None else [], **kwargs}, outfile)
 
     @staticmethod
     def load_tokens(path: Union[str, Path, PurePath]) -> Union[List[Any], Dict]:
@@ -597,24 +597,25 @@ class MIDITokenizer(ABC):
                        'beat_res': {f'{k1}_{k2}': v for (k1, k2), v in self.beat_res.items()},
                        'nb_velocities': len(self.velocities),
                        'additional_tokens': self.additional_tokens,
+                       '_pad': self._pad,
                        '_sos_eos': self._sos_eos,
                        '_mask': self._mask,
-                       'encoding': self.__class__.__name__}, outfile, indent=4)
+                       'encoding': self.__class__.__name__,
+                       'miditok_version': CURRENT_PACKAGE_VERSION}, outfile, indent=4)
 
-    def load_params(self, params: Union[str, Path, PurePath, Dict[str, Any]]):
+    def load_params(self, params: Union[str, Path, PurePath]):
         r"""Load parameters and set the encoder attributes
 
-        :param params: can be a path to the parameter (json encoded) file or a dictionary
+        :param params: can be a path to the parameter (json encoded) file
         """
-        if isinstance(params, (str, Path, PurePath)):
-            with open(params) as param_file:
-                params = json.load(param_file)
+        with open(params) as param_file:
+            params = json.load(param_file)
 
         if not isinstance(params['pitch_range'], range):
             params['pitch_range'] = range(*params['pitch_range'])
 
         for key, value in params.items():
-            if key == 'encoding':
+            if key in ['encoding', 'miditok_version']:
                 continue
             elif key == 'beat_res':
                 value = {tuple(map(int, beat_range.split('_'))): res for beat_range, res in value.items()}
@@ -622,10 +623,12 @@ class MIDITokenizer(ABC):
                 value['TimeSignature'] = value.get('TimeSignature', False)
             setattr(self, key, value)
 
-        # when loading from params of miditok < v1.2.0
-        if '_sos_eos' not in params:
+        # when loading from params of miditok of previous versions
+        if '_pad' not in params:  # miditok < v1.3.0
+            self._pad = False
+        if '_sos_eos' not in params:  # miditok < v1.2.0
             self._sos_eos = False
-        if '_mask' not in params:
+        if '_mask' not in params:  # miditok < v1.2.0
             self._mask = False
 
     def __call__(self, midi: MidiFile, *args, **kwargs):
