@@ -14,7 +14,8 @@ from miditoolkit import MidiFile, Instrument, Note, TempoChange
 from .midi_tokenizer_base import MIDITokenizer
 from .vocabulary import Vocabulary, Event
 from .utils import detect_chords
-from .constants import *
+from .constants import PITCH_RANGE, NB_VELOCITIES, BEAT_RES, ADDITIONAL_TOKENS, TIME_DIVISION, TEMPO, \
+    MIDI_INSTRUMENTS, CHORD_MAPS, CURRENT_PACKAGE_VERSION
 
 
 # recommended range from the GM2 specs
@@ -33,14 +34,17 @@ class MuMIDI(MIDITokenizer):
             The values are the resolution, in samples per beat, of the given range, ex 8
     :param nb_velocities: number of velocity bins
     :param additional_tokens: specifies additional tokens (chords, time signature, rests, tempo)
-    :param sos_eos_tokens: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary
+    :param pad: will include a PAD token, used when training a model with batch of sequences of
+            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
+    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
+            (default: False)
     :param mask: will add a MASK token to the vocabulary (default: False)
     :param params: can be a path to the parameter (json encoded) file or a dictionary
     :param drum_pitch_range: range of used MIDI pitches for drums exclusively
     """
     def __init__(self, pitch_range: range = PITCH_RANGE, beat_res: Dict[Tuple[int, int], int] = BEAT_RES,
                  nb_velocities: int = NB_VELOCITIES, additional_tokens: Dict[str, bool] = ADDITIONAL_TOKENS,
-                 programs: List[int] = None, sos_eos_tokens: bool = False, mask: bool = False, params=None,
+                 programs: List[int] = None, pad: bool = True, sos_eos: bool = False, mask: bool = False, params=None,
                  drum_pitch_range: range = DRUM_PITCH_RANGE):
         additional_tokens['Rest'] = False
         additional_tokens['TimeSignature'] = False  # not compatible
@@ -48,7 +52,7 @@ class MuMIDI(MIDITokenizer):
         self.programs = list(range(-1, 128)) if programs is None else programs
         # used in place of positional encoding
         self.max_bar_embedding = 60  # this attribute might increase during encoding
-        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, sos_eos_tokens, mask, params)
+        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, pad, sos_eos, mask, params)
 
     def save_params(self, out_dir: Union[str, Path, PurePath]):
         r"""Override the parent class method to include additional parameter drum pitch range
@@ -66,10 +70,11 @@ class MuMIDI(MIDITokenizer):
                        'nb_velocities': len(self.velocities),
                        'additional_tokens': self.additional_tokens,
                        'encoding': self.__class__.__name__,
+                       'miditok_version': CURRENT_PACKAGE_VERSION,
                        'max_bar_embedding': self.max_bar_embedding},
                       outfile)
 
-    def midi_to_tokens(self, midi: MidiFile) -> List[List[int]]:
+    def midi_to_tokens(self, midi: MidiFile, *args, **kwargs) -> List[List[int]]:
         r"""Override the parent class method
         Converts a MIDI file in a tokens representation, a sequence of "time steps".
         A time step is a list of tokens where:
@@ -182,7 +187,7 @@ class MuMIDI(MIDITokenizer):
 
     def track_to_tokens(self, track: Instrument) -> List[List[Union[Event, int]]]:
         r"""Converts a track (miditoolkit.Instrument object) into a sequence of tokens
-        For each note, it create a time step as a list of tokens where:
+        For each note, it creates a time step as a list of tokens where:
             (list index: token type)
             0: Pitch (as an Event object for sorting purpose afterwards)
             1: Velocity
@@ -201,11 +206,11 @@ class MuMIDI(MIDITokenizer):
             duration = note.end - note.start
             dur_idx = np.argmin(np.abs(dur_bins - duration))
             if not track.is_drum:
-                tokens.append([Event(type_='Pitch', time=note.start, value=note.pitch, desc=track.program),
+                tokens.append([Event(type_='Pitch', value=note.pitch, time=note.start, desc=track.program),
                                self.vocab[1].event_to_token[f'Velocity_{note.velocity}'],
                                self.vocab[2].event_to_token[f'Duration_{".".join(map(str, self.durations[dur_idx]))}']])
             else:
-                tokens.append([Event(type_='DrumPitch', time=note.start, value=note.pitch, desc=-1),
+                tokens.append([Event(type_='DrumPitch', value=note.pitch, time=note.start, desc=-1),
                                self.vocab[1].event_to_token[f'Velocity_{note.velocity}'],
                                self.vocab[2].event_to_token[f'Duration_{".".join(map(str, self.durations[dur_idx]))}']])
 
@@ -315,9 +320,9 @@ class MuMIDI(MIDITokenizer):
         :return: the vocabulary object
         """
         if sos_eos_tokens is not None:
-            print(f'\033[93msos_eos_tokens argument is depreciated and will be removed in a future update, '
-                  f'_create_vocabulary now uses self._sos_eos attribute set a class init \033[0m')
-        vocab = [Vocabulary({'PAD_None': 0}, sos_eos=self._sos_eos, mask=self._mask) for _ in range(5)]
+            print('\033[93msos_eos_tokens argument is depreciated and will be removed in a future update, '
+                  '_create_vocabulary now uses self._sos_eos attribute set a class init \033[0m')
+        vocab = [Vocabulary(pad=self._pad, sos_eos=self._sos_eos, mask=self._mask) for _ in range(5)]
 
         # PITCH
         vocab[0].add_event(f'Pitch_{i}' for i in self.pitch_range)
@@ -348,7 +353,7 @@ class MuMIDI(MIDITokenizer):
 
         # TEMPO
         if self.additional_tokens['Tempo']:
-            vocab.append(Vocabulary({'PAD_None': 0}, sos_eos=self._sos_eos, mask=self._mask))
+            vocab.append(Vocabulary(pad=self._pad, sos_eos=self._sos_eos, mask=self._mask))
             vocab[-1].add_event(f'Tempo_{i}' for i in self.tempos)
 
         # PROGRAM
