@@ -13,7 +13,8 @@ from miditoolkit import MidiFile, Instrument, Note, TempoChange, TimeSignature
 
 from .midi_tokenizer_base import MIDITokenizer
 from .vocabulary import Vocabulary, Event
-from .constants import *
+from .constants import PITCH_RANGE, NB_VELOCITIES, BEAT_RES, ADDITIONAL_TOKENS, TIME_DIVISION, TIME_SIGNATURE, TEMPO, \
+    MIDI_INSTRUMENTS, CURRENT_PACKAGE_VERSION
 
 
 class Octuple(MIDITokenizer):
@@ -27,19 +28,22 @@ class Octuple(MIDITokenizer):
             The values are the resolution, in samples per beat, of the given range, ex 8
     :param nb_velocities: number of velocity bins
     :param additional_tokens: specifies additional tokens (time signature, tempo)
-    :param sos_eos_tokens: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary
+    :param pad: will include a PAD token, used when training a model with batch of sequences of
+            unequal lengths, and usually at index 0 of the vocabulary. (default: True)
+    :param sos_eos: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary.
+            (default: False)
     :param mask: will add a MASK token to the vocabulary (default: False)
     :param params: can be a path to the parameter (json encoded) file or a dictionary
     """
     def __init__(self, pitch_range: range = PITCH_RANGE, beat_res: Dict[Tuple[int, int], int] = BEAT_RES,
                  nb_velocities: int = NB_VELOCITIES, additional_tokens: Dict[str, bool] = ADDITIONAL_TOKENS,
-                 programs: List[int] = None, sos_eos_tokens: bool = False, mask: bool = False, params=None):
+                 programs: List[int] = None, pad: bool = True, sos_eos: bool = False, mask: bool = False, params=None):
         additional_tokens['Chord'] = False  # Incompatible additional token
         additional_tokens['Rest'] = False
         # used in place of positional encoding
         self.programs = list(range(-1, 128)) if programs is None else programs
         self.max_bar_embedding = 60  # this attribute might increase during encoding
-        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, sos_eos_tokens, mask, params)
+        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, pad, sos_eos, mask, params)
 
     def save_params(self, out_dir: Union[str, Path, PurePath]):
         r"""Override the parent class method to include additional parameter drum pitch range
@@ -58,11 +62,12 @@ class Octuple(MIDITokenizer):
                        '_sos_eos': self._sos_eos,
                        '_mask': self._mask,
                        'encoding': self.__class__.__name__,
+                       'miditok_version': CURRENT_PACKAGE_VERSION,
                        'programs': self.programs,
                        'max_bar_embedding': self.max_bar_embedding},
                       outfile)
 
-    def midi_to_tokens(self, midi: MidiFile) -> List[List[int]]:
+    def midi_to_tokens(self, midi: MidiFile, *args, **kwargs) -> List[List[int]]:
         r"""Override the parent class method
         Converts a MIDI file in a tokens representation, a sequence of "time steps".
         A time step is a list of tokens where:
@@ -161,7 +166,7 @@ class Octuple(MIDITokenizer):
             # Note attributes
             duration = note.end - note.start
             dur_index = np.argmin(np.abs(dur_bins - duration))
-            event = [Event(type_='Pitch', time=note.start, value=note.pitch,
+            event = [Event(type_='Pitch', value=note.pitch, time=note.start,
                            desc=-1 if track.is_drum else track.program),
                      self.vocab[1].event_to_token[f'Velocity_{note.velocity}'],
                      self.vocab[2].event_to_token[f'Duration_{".".join(map(str, self.durations[dur_index]))}'],
@@ -268,8 +273,8 @@ class Octuple(MIDITokenizer):
             program = int(time_step[3].value)
             current_pos = int(time_step[4].value)
             current_bar = int(time_step[5].value)
-            current_tick = current_time_sig_tick + (current_bar - current_time_sig_bar) * ticks_per_bar \
-                           + current_pos * ticks_per_sample
+            current_tick = (current_time_sig_tick + (current_bar - current_time_sig_bar) * ticks_per_bar +
+                            current_pos * ticks_per_sample)
 
             # Append the created note
             tracks[program].append(Note(vel, pitch, current_tick, current_tick + duration))
@@ -332,9 +337,9 @@ class Octuple(MIDITokenizer):
         :return: the vocabulary object
         """
         if sos_eos_tokens is not None:
-            print(f'\033[93msos_eos_tokens argument is depreciated and will be removed in a future update, '
-                  f'_create_vocabulary now uses self._sos_eos attribute set a class init \033[0m')
-        vocab = [Vocabulary({'PAD_None': 0}, sos_eos=self._sos_eos, mask=self._mask) for _ in range(6)]
+            print('\033[93msos_eos_tokens argument is depreciated and will be removed in a future update, '
+                  '_create_vocabulary now uses self._sos_eos attribute set a class init \033[0m')
+        vocab = [Vocabulary(pad=self._pad, sos_eos=self._sos_eos, mask=self._mask) for _ in range(6)]
 
         # PITCH
         vocab[0].add_event(f'Pitch_{i}' for i in self.pitch_range)
@@ -357,12 +362,12 @@ class Octuple(MIDITokenizer):
 
         # TEMPO
         if self.additional_tokens['Tempo']:
-            vocab.append(Vocabulary({'PAD_None': 0}, sos_eos=self._sos_eos, mask=self._mask))
+            vocab.append(Vocabulary(pad=self._pad, sos_eos=self._sos_eos, mask=self._mask))
             vocab[-1].add_event(f'Tempo_{i}' for i in self.tempos)
 
         # TIME_SIGNATURE
         if self.additional_tokens['TimeSignature']:
-            vocab.append(Vocabulary({'PAD_None': 0}, sos_eos=self._sos_eos, mask=self._mask))
+            vocab.append(Vocabulary(pad=self._pad, sos_eos=self._sos_eos, mask=self._mask))
             vocab[-1].add_event(f'TimeSig_{i[0]}/{i[1]}' for i in self.time_signatures)
 
         return vocab
